/*
 * Copyright © 2007 Jerome Glisse <glisse@freedesktop.org>
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 */
#ifndef DUMP_H
#define DUMP_H

#include <stdio.h>
#include "list.h"

struct mmreg
{
    struct list_head    list;
    unsigned            offset;
    unsigned            value;
};

struct ixreg
{
    struct list_head    list;
    unsigned            index;
    unsigned            data;
    unsigned            offset;
    unsigned            value;
};

struct dump
{
    unsigned            pciid;
    unsigned            cmmregs;
    unsigned            cixregs;
    struct list_head    mmregs;
    struct list_head    ixregs;
};

void dump_init(struct dump *dump);
void dump_fini(struct dump *dump);
void dump_add_mmreg(struct dump *dump, struct mmreg *mmreg);
void dump_add_ixreg(struct dump *dump, struct ixreg *ixreg);
void dump_print(struct dump *dump, FILE *file);
struct mmreg *dump_find_mmreg(struct dump *dump, struct mmreg *mmreg);
struct ixreg *dump_find_ixreg(struct dump *dump, struct ixreg *ixreg);

struct mmreg *mmreg_new(unsigned o, unsigned v);
struct ixreg *ixreg_new(unsigned idx, unsigned ida, unsigned o, unsigned v);

int dump_xml_read(struct dump *dump, const char *filename);
int dump_xml_write(struct dump *dump, const char *filename);

#endif
