/*
 * Copyright 2008 Jerome Glisse <glisse@freedesktop.org>
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee,
 * provided that the above copyright notice appear in all copies and that
 * both that this copyright notice and this permission notice appear in
 * supporting electronic documentation.
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR AUTHORS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 * SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
#ifndef RADEON_H
#define RADEON_H

#include <stdint.h>
#include <pciaccess.h>
#include "dump.h"

struct radeon;
typedef uint32_t    u32;

enum radeon_family {
    CHIP_R100,
    CHIP_RV100,
    CHIP_RS100,
    CHIP_RV200,
    CHIP_RS200,
    CHIP_R200,
    CHIP_RV250,
    CHIP_RS300,
    CHIP_RV280,
    CHIP_R300,
    CHIP_R350,
    CHIP_RV350,
    CHIP_RV380,
    CHIP_R420,
    CHIP_R423,
    CHIP_RV410,
    CHIP_RS400,
    CHIP_RS480,
    CHIP_RS600,
    CHIP_RS690,
    CHIP_RS740,
    CHIP_RV515,
    CHIP_R520,
    CHIP_RV530,
    CHIP_RV560,
    CHIP_RV570,
    CHIP_R580,
    CHIP_R600,
    CHIP_RV610,
    CHIP_RV630,
    CHIP_RV670,
    CHIP_RV620,
    CHIP_RV635,
    CHIP_RS780,
    CHIP_RS880,
    CHIP_RV770,
    CHIP_RV730,
    CHIP_RV710,
    CHIP_RV740,
    CHIP_LAST,
};

struct radeon_asic
{
    int (*mmrd32)(struct radeon*, struct mmreg*);
    int (*mmwr32)(struct radeon*, struct mmreg*);
    int (*ixrd32)(struct radeon*, struct ixreg*);
    int (*ixwr32)(struct radeon*, struct ixreg*);
};

struct radeon_pciid
{
    unsigned                    vendor;
    unsigned                    device;
    enum radeon_family          family;
    char                        name[32];
    struct radeon_asic          *asic;
};

struct radeon
{
    struct pci_device           *dev;
    unsigned char * volatile    mmio;
    unsigned char * volatile    vram;
    pciaddr_t                   mmio_base;
    pciaddr_t                   mmio_size;
    pciaddr_t                   vram_base;
    pciaddr_t                   vram_size;
    uint8_t                     *bios;
    const struct radeon_pciid   *pciid;
};

int radeon_probe(struct radeon *radeon);
void radeon_close(struct radeon *radeon);
u32 radeon_mmio_rd32(struct radeon *radeon, u32 offset);
void radeon_mmio_wr32(struct radeon *radeon, u32 offset, u32 value);
int radeon_read_bios(struct radeon *radeon, const char *filename);
const struct radeon_pciid *radeon_find_family(unsigned vendor, unsigned device);

int rv515_mmrd32(struct radeon *radeon, struct mmreg *mmreg);
int rv515_mmwr32(struct radeon *radeon, struct mmreg *mmreg);
int rv515_ixrd32(struct radeon *radeon, struct ixreg *ixreg);
int rv515_ixwr32(struct radeon *radeon, struct ixreg *ixreg);

#endif
