/*
 * Copyright © 2007 Jerome Glisse <glisse@freedesktop.org>
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of version 2 of the GNU General Public License
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 */
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <pciaccess.h>
#include "radeon.h"

static void usage(void)
{
    printf("Usage: radeonvram\n");
    exit(-1);
}

int vram_test_hdp(struct radeon *radeon)
{
    u32 *vram = (u32*)&radeon->vram[6 * 1024 * 1024];
    /* Write somethings at 6M (all radeon should have enough memory) */
    vram[0] = 0xDEADBEEF;
    vram[1] = 0xCAFEDEAD;
    usleep(500);
    if (vram[0] != 0xDEADBEEF || vram[1] != 0xCAFEDEAD) {
        printf("%s failed\n", __func__);
        return -1;
    }
    printf("%s succeed\n", __func__);
    return 0;
}

int vram_test_gpu(struct radeon *radeon)
{
    u32 vram0, vram1, addr;

    addr = 6 * 1024 * 1024;
    /* Write somethings at 6M (all radeon should have enough memory) */
    radeon_mmio_wr32(radeon, 0x0, (1 << 31) | addr);
    radeon_mmio_wr32(radeon, 0x4, 0xDEADBEEF);
    radeon_mmio_wr32(radeon, 0x0, (1 << 31) | (addr + 4));
    radeon_mmio_wr32(radeon, 0x4, 0xCAFEDEAD);
    usleep(500);
    radeon_mmio_wr32(radeon, 0x0, (1 << 31) | addr);
    vram0 = radeon_mmio_rd32(radeon, 0x4);
    radeon_mmio_wr32(radeon, 0x0, (1 << 31) | (addr + 4));
    vram1 = radeon_mmio_rd32(radeon, 0x4);
    if (vram0 != 0xDEADBEEF || vram1 != 0xCAFEDEAD) {
        printf("%s failed\n", __func__);
        return -1;
    }
    printf("%s succeed\n", __func__);
    return 0;
}

int main(int argc, char *argv[])
{
    struct radeon radeon;

    if (argc > 1) {
        usage();
    }
    if (radeon_probe(&radeon)) {
        fprintf(stderr, "Found no graphics card (dumb tool)!\n");
        return 1;
    }
    printf("BUS_CNTL:       0x%08X\n", radeon_mmio_rd32(&radeon, 0x4C));
    printf("CONFIG_CNTL:    0x%08X\n", radeon_mmio_rd32(&radeon, 0xE0));
    printf("CONFIG_MEMSIZE: 0x%08X\n", radeon_mmio_rd32(&radeon, 0xF8));
    printf("COMMAND|STATUS: 0x%08X\n", radeon_mmio_rd32(&radeon, 0x5004));
    vram_test_hdp(&radeon);
    vram_test_gpu(&radeon);
out:
    radeon_close(&radeon);
    return 0;
}

